import os
from genesys.app.services import files_service, svn_service
from genesys.app.config import SVN_PARENT_PATH, SVN_PARENT_URL, TEMPLATE_FILES_DIR
from configparser import NoOptionError
from genesys.app.utils import config_helpers
from configparser import ConfigParser
from genesys.app.config import (SVN_PARENT_PATH,
                                SVN_PARENT_URL,
                                FILE_MAP,
                                LOGIN_NAME)

def create_task_file(project_name, base_file_directory, base_svn_directory, all_persons, task_type):
    file_map_parser = ConfigParser()
    acl_parser = ConfigParser()
    svn_authz_path = os.path.join(SVN_PARENT_PATH, project_name, 'conf/authz')
    blend_file_url = os.path.join(SVN_PARENT_URL, base_file_directory)
    all_users = [person[LOGIN_NAME] for person in all_persons]
    file_map_url = os.path.join(SVN_PARENT_URL, project_name, '.conf/file_map')

    config_helpers.load_file_map(file_map_url, file_map_parser)
    config_helpers.load_config(svn_authz_path, acl_parser)


    file_folder_url = os.path.dirname(blend_file_url)
    try:
        task_type_map = file_map_parser.get('file_map', task_type).lower()
        if task_type_map == 'base':
            if not svn_service.is_svn_url(file_folder_url):
                svn_service.svn_make_dirs(file_folder_url, log_message=f'created {file_folder_url}')
            if not svn_service.is_svn_url(f'{blend_file_url}.blend'):
                svn_service.svn_import(path=os.path.join(TEMPLATE_FILES_DIR,'blender.blend'),
                                        repo_url=f'{blend_file_url}.blend',
                                        log_message=f'create {blend_file_url}.blend')
        elif task_type_map == 'none':
            pass
        else:
            if not svn_service.is_svn_url(file_folder_url):
                svn_service.svn_make_dirs(file_folder_url, log_message=f'created {file_folder_url}')
            if not svn_service.is_svn_url(f'{blend_file_url}_{task_type_map}.blend'):
                svn_service.svn_import(path=os.path.join(TEMPLATE_FILES_DIR,'blender.blend'),
                                        repo_url=f'{blend_file_url}_{task_type_map}.blend',
                                        log_message=f'create {blend_file_url}.blend')
    except NoOptionError:
        if not svn_service.is_svn_url(file_folder_url):
            svn_service.svn_make_dirs(file_folder_url, log_message=f'created {file_folder_url}')
        if not svn_service.is_svn_url(f'{blend_file_url}_{task_type}.blend'):
            svn_service.svn_import(path=os.path.join(TEMPLATE_FILES_DIR,'blender.blend'),
                                        repo_url=f'{blend_file_url}_{task_type}.blend',
                                        log_message=f'create {blend_file_url}.blend')
        file_map_parser.set('file_map', task_type, task_type)
    

    #TODO create acl fo new task
    create_new_task_acl(all_users=all_users,
                        base_svn_directory=base_svn_directory,
                        acl_parser=acl_parser,
                        file_map_parser=file_map_parser,
                        task_type=task_type)

    config_helpers.write_file_map(file_map_url, file_map_parser)
    config_helpers.write_config(svn_authz_path, acl_parser)

def create_new_task_acl(all_users, base_svn_directory, acl_parser, file_map_parser, task_type):
    try:
        task_type_map = file_map_parser.get('file_map', task_type).lower()
        if task_type_map == 'base':
            svn_directory = f'{base_svn_directory}.blend'
            if svn_directory in acl_parser:
                pass
            else:
                for user in all_users:
                    if svn_directory in acl_parser:
                        acl_parser.set(svn_directory, user, '')
                    else:
                        acl_parser[svn_directory] = {
                            '@admin':'rw',
                            user:''
                        }
        elif task_type_map == 'none':
            pass
        else:
            svn_directory = f'{base_svn_directory}_{task_type_map}.blend'
            if svn_directory in acl_parser:
                pass
            else:
                for user in all_users:
                    if svn_directory in acl_parser:
                        acl_parser.set(svn_directory, user, '')
                    else:
                        acl_parser[svn_directory] = {
                            '@admin':'rw',
                            user:''
                        }
    except NoOptionError:
        svn_directory = f'{base_svn_directory}_{task_type}.blend'
        if svn_directory in acl_parser:
            pass
        else:
            for user in all_users:
                if svn_directory in acl_parser:
                    acl_parser.set(svn_directory, user, '')
                else:
                    acl_parser[svn_directory] = {
                        '@admin':'rw',
                        user:''
                    }

def delete_task_file(project_name, base_file_directory, task_type):
    file_map_parser = ConfigParser()
    acl_parser = ConfigParser()
    svn_authz_path = os.path.join(SVN_PARENT_PATH, project_name, 'conf/authz')
    # if asset_task['entity_type'] == 'asset':
    #     new_blend_file_url = os.path.join(os.path.dirname(blend_file_url), asset_task['new_file_name'])
    # elif asset_task['entity_type'] == 'shot':
    #     shot_folder = os.path.join(os.path.dirname(os.path.dirname(blend_file_url)), \
    #         asset_task['new_file_name'].rsplit('_', 1)[1])
    #     new_blend_file_url = os.path.join(shot_folder, asset_task['new_file_name'])
    #     print('shot', new_blend_file_url)
    #     print(shot_folder)
    blend_file_url = os.path.join(SVN_PARENT_URL, base_file_directory)
    file_map_url = os.path.join(SVN_PARENT_URL, project_name, '.conf/file_map')

    config_helpers.load_file_map(file_map_url, file_map_parser)
    # config_helpers.load_config(svn_authz_path, acl_parser)

    try:
        task_type_map = file_map_parser.get('file_map', task_type).lower()
        if task_type_map == 'base':
            if svn_service.is_svn_url(f'{blend_file_url}.blend'):
                svn_service.svn_delete(f'{blend_file_url}.blend', log_message=f'created {blend_file_url}.blend')
        elif task_type_map == 'none':
            pass
        else:
            if svn_service.is_svn_url(f'{blend_file_url}_{task_type_map}.blend'):
                svn_service.svn_delete(f'{blend_file_url}_{task_type_map}.blend', log_message=f'created {blend_file_url}_{task_type_map}.blend')
    except NoOptionError:
        if svn_service.is_svn_url(f'{blend_file_url}_{task_type}.blend'):
            svn_service.svn_delete(f'{blend_file_url}_{task_type}.blend', log_message=f'created {blend_file_url}_{task_type}.blend')
        # file_map_parser.set('file_map', task_type, task_type)
    
    # config_helpers.write_file_map(file_map_url, file_map_parser)
    # config_helpers.write_config(svn_authz_path, acl_parser)


from flask import Blueprint, jsonify, request
from flask_restful import Api, Resource
from genesys.app.config import SVN_PARENT_PATH, SVN_PARENT_URL, FILE_MAP
from genesys.app.blueprints.project.utils import create_project_folders, create_project_file_map
from genesys.app.services import queue_store
import os
from genesys.app.utils import config_helpers
from genesys.app.blueprints.asset.utils import rename_asset_task_file, rename_task_acl_section
from configparser import ConfigParser
from genesys.app import config

asset = Blueprint('asset', __name__)
api = Api(asset)

def rename_asset(data, project_name):
    for asset_task in data:
        file_map_parser = ConfigParser()
        acl_parser = ConfigParser()
        task_type = asset_task['task_type']
        project = asset_task['project']
        root = os.path.join(project['file_tree']['working']['mountpoint'], project['file_tree']['working']['root'], '')
        # replacing file tree mount point with genesys config mount point
        #TODO rename shot folder
        base_file_directory = asset_task['working_file_path'].split(root,1)[1]
        base_svn_directory = asset_task['base_svn_directory']

        blend_file_url = os.path.join(SVN_PARENT_URL, base_file_directory)
        if asset_task['entity_type'] == 'asset':
            new_blend_file_url = os.path.join(os.path.dirname(blend_file_url), asset_task['new_file_name'])
        elif asset_task['entity_type'] == 'shot':
            shot_folder = os.path.join(os.path.dirname(os.path.dirname(blend_file_url)), \
                asset_task['new_file_name'].rsplit('_', 1)[1])
            new_blend_file_url = os.path.join(shot_folder, asset_task['new_file_name'])
        new_base_svn_directory = os.path.join(os.path.dirname(base_svn_directory), asset_task['new_file_name'])
        svn_authz_path = os.path.join(SVN_PARENT_PATH, project_name, 'conf/authz')
        file_map_url = os.path.join(SVN_PARENT_URL, project_name, '.conf/file_map')

        config_helpers.load_file_map(file_map_url, file_map_parser)
        config_helpers.load_config(svn_authz_path, acl_parser)
        rename_asset_task_file(
            old_blend_file_url=blend_file_url,
            new_blend_file_url=new_blend_file_url, 
            file_map_parser=file_map_parser, 
            task_type=task_type)
        rename_task_acl_section(
            old_base_svn_directory= base_svn_directory,
            base_svn_directory= new_base_svn_directory,
            file_map_parser= file_map_parser,
            acl_parser= acl_parser,
            task_type= task_type
            )
        config_helpers.write_file_map(file_map_url=file_map_url, file_map_parser=file_map_parser)
        config_helpers.write_config(svn_authz_path, acl_parser)

class Asset(Resource):
    def put(self, project_name):
        data = request.get_json()
        project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
        if config.ENABLE_JOB_QUEUE:
            queue_store.job_queue.enqueue(
                rename_asset,
                args=(data, project_name),
                job_timeout=10,
            )
            return {"job": "running"}
        else:
            rename_asset(data, project_name)
            return jsonify(message=f'project created', project_name=project_name, svn_url=project_repo_url)

api.add_resource(Asset, '/asset/<string:project_name>')




from flask import Blueprint, jsonify, request
from flask_restful import Api, Resource
from genesys.app.config import (SVN_PARENT_PATH,
                                SVN_PARENT_URL,
                                FILE_MAP,
                                LOGIN_NAME)
from genesys.app.blueprints.task.utils import create_task_file, create_new_task_acl, delete_task_file
from genesys.app.services import svn_service, queue_store
from genesys.app.utils import config_helpers
import os
from configparser import ConfigParser
from genesys.app import config

task = Blueprint('task', __name__)
api = Api(task)

def update_svn_acl(data, project_name):
    file_map_parser = ConfigParser()
    acl_parser = ConfigParser()

    file_map_url = os.path.join(SVN_PARENT_URL, project_name, '.conf/file_map')
    svn_authz_path = os.path.join(SVN_PARENT_PATH, project_name, 'conf/authz')
    config_helpers.load_file_map(file_map_url, file_map_parser)
    config_helpers.load_config(svn_authz_path, acl_parser)

    svn_service.svn_update_acl(
        base_svn_directory=data['base_svn_directory'],
        acl_parser=acl_parser,
        file_map_parser=file_map_parser,
        task_type=data['task_type'],
        person=data['person'][LOGIN_NAME],
        permission=data['permission']
    )
    if data['permission'] == 'rw':
        for i in data['dependencies']:
            svn_service.svn_update_acl(
                base_svn_directory=i,
                acl_parser=acl_parser,
                file_map_parser=file_map_parser,
                task_type='modeling',
                person=data['person'][LOGIN_NAME],
                permission='r'
            )
    elif data['permission'] == 'none':
        for i in data['dependencies']:
            svn_service.svn_update_acl(
                base_svn_directory=i,
                acl_parser=acl_parser,
                file_map_parser=file_map_parser,
                task_type='modeling',
                person=data['person'][LOGIN_NAME],
                permission='d'
            )

    config_helpers.write_file_map(file_map_url, file_map_parser)
    config_helpers.write_config(svn_authz_path, acl_parser)

class Task(Resource):
    def post(self, project_name):
        data = request.get_json()
        project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
        project = (data['project'])
        root = os.path.join(project['file_tree']['working']['mountpoint'], project['file_tree']['working']['root'],'')
        # replacing file tree mount point with genesys config mount point
        base_file_directory = data['base_file_directory'].split(root,1)[1]
        if config.ENABLE_JOB_QUEUE:
            queue_store.job_queue.enqueue(
                create_task_file,
                args=(project_name, base_file_directory, data['base_svn_directory'], data['all_persons'], data['task_type']),
                job_timeout=10,
            )
            return {"job": "running"}
        else:
            create_task_file(project_name=project_name,
                                base_file_directory=base_file_directory,
                                base_svn_directory=data['base_svn_directory'],
                                all_persons=data['all_persons'],
                                task_type=data['task_type'],
                                )
            return jsonify(message=f'task created')

    def delete(self, project_name):
        data = request.get_json()
        project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
        project = (data['project'])
        root = os.path.join(project['file_tree']['working']['mountpoint'], project['file_tree']['working']['root'],'')
        # replacing file tree mount point with genesys config mount point
        base_file_directory = data['base_file_directory'].split(root,1)[1]
        if config.ENABLE_JOB_QUEUE:
            queue_store.job_queue.enqueue(
                delete_task_file,
                args=(project_name, base_file_directory, data['task_type']),
                job_timeout=10,
            )
            return {"job": "running"}
        else:
            delete_task_file(project_name, base_file_directory, task_type=data['task_type'])
            return jsonify(message=f'task deleted')


class Task_File_Access_Control(Resource):
    def put(self, project_name):
        data = request.get_json()
        if config.ENABLE_JOB_QUEUE:
            queue_store.job_queue.enqueue(
                update_svn_acl,
                args=(data, project_name),
                job_timeout=10,
            )
            return {"job": "running"}
        else:
            update_svn_acl(data, project_name)
            project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
            return jsonify(message=f'project created', project_name=project_name, svn_url=project_repo_url)


api.add_resource(Task, '/task/<string:project_name>')
api.add_resource(Task_File_Access_Control, '/task_acl/<string:project_name>')